import os
import datetime
import subprocess
import multiprocessing

EXCLUDES =("Mk", "Tools", "Templates", "Keywords", "distfiles", "packages",
           "CVS", ".svn")
CAT = "cat"
MAKE = "make"
PORTSDIR = os.getenv("PORTSDIR")
if not PORTSDIR:
    PORTSDIR = "/usr/ports"

def get_ports(strmap):
    portlist = []
    for kv in strmap.strip().split():
        kpd = kv.split(":") # <whatever>:<portsdir>[:whatever]
        basep, category, port = kpd[1].rsplit(os.path.sep, 2)
        portlist.append(os.path.join(category, port))
    return portlist

def get_run_depends(category, port, pdir):
    cmd = [MAKE, "-C", pdir, "-VRUN_DEPENDS"]
    output = subprocess.check_output(cmd).strip()
    return get_ports(output)

def get_build_depends(category, port, pdir):
    cmd = [MAKE, "-C", pdir, "-VBUILD_DEPENDS"]
    output = subprocess.check_output(cmd).strip()
    return get_ports(output)

def get_lib_depends(category, port, pdir):
    cmd = [MAKE, "-C", pdir, "-VLIB_DEPENDS"]
    output = subprocess.check_output(cmd).strip()
    return get_ports(output)

def get_website(category, port, pdir):
    cmd = [MAKE, "-C", pdir, "www-site"]
    output = subprocess.check_output(cmd).strip()
    return output.strip()

def walk_category(cat):
    portlist = {}
    catdir = os.path.join(PORTSDIR, cat)
    for port in os.listdir(catdir):
        if port in ("CVS", ".svn"):
            continue
        pdir = os.path.join(catdir, port)
        if not os.path.isdir(pdir):
            continue
        print("Creating entry for %s/%s" % (cat, port))
        entry = {
            "run_depends": get_run_depends(cat, port, pdir),
            "lib_depends": get_lib_depends(cat, port, pdir),
            "build_depends": get_build_depends(cat, port, pdir),
            "using": 0
            # "website": get_website(cat, port, pdir),
            # "description":
        }
        portlist[os.path.join(cat, port)] = entry
    return portlist

if __name__ == "__main__":
    categories = os.listdir(PORTSDIR)
    _isdir = lambda x: os.path.isdir(os.path.join(PORTSDIR, x))
    categories = [c for c in categories if _isdir(c) and c not in EXCLUDES]
    pool = multiprocessing.Pool()
    results = pool.map(walk_category, categories)
    allports = {}
    for res in results:
        allports.update(res)
    for name, entry in allports.viewitems():
        for k in ("run_depends", "lib_depends", "build_depends"):
            for dep in entry[k]:
                if dep in allports:
                    allports[dep]["using"] += 1
                else:
                    print("invalid reference '%s' for port '%s'" % (dep, name))
    with open("portsdb.js", "w") as fp:
        fp.write('{ "date": "%s", "categories": %s, "list": {\n' %
                 (datetime.date.today(), str(categories).replace("'", "\"")))
        for name, entry in allports.viewitems():
            rdeps = str(entry["run_depends"]).replace("'", "\"")
            ldeps = str(entry["lib_depends"]).replace("'", "\"")
            bdeps = str(entry["build_depends"]).replace("'", "\"")
            fp.write('"%s": { "run_depends": %s, "lib_depends": %s, "build_depends": %s, "using": %d },\n' % (name, rdeps, ldeps, bdeps , entry["using"]))
        fp.seek(-2, os.SEEK_CUR)
        fp.write('}, "count": %d }' % len(allports))
    print("Done")
